from Bio import SeqIO
from flask import Flask, request, jsonify
from design import design_diagnostic_digest
from flask_cors import CORS
from enzymes import ALL_ENZYMES

import io
import pathlib

import traceback

__version__ = open("VERSION").read().strip()

app = Flask(__name__)
app.config["MAX_CONTENT_LENGTH"] = 16 * 1000

CORS(app)

@app.route("/design", methods=["POST"])
def design():
    plasmid_file = request.files.get("plasmid")
    if plasmid_file is None:
        return jsonify({
            "linear": [],
            "multi": []
        })


    try:
        enzymes = request.form.get("enzymes")
        enzymes = set(map(lambda x: x.strip(), enzymes.split(",")))

        invalid = enzymes - ALL_ENZYMES

        if len(invalid) != 0:
            raise Exception(f"invalid enzymes: {', '.join(invalid)}")

    except Exception as e:
        return jsonify({
            "error": str(e)
        })

    try:
        plasmid_io = io.StringIO(plasmid_file.read().decode("utf8"))
        fname = plasmid_file.filename
        fext = pathlib.Path(fname).suffix.lstrip(".")
        print(fext)

        ext = None

        if fext in {"ape", "gb", "genbank", "gbk"}:
            ext = "gb"
        elif fext in {"fasta"}:
            ext = "fasta"
        
        if not ext:
            raise Exception("unknown file type")

        plasmid = SeqIO.read(plasmid_io, ext)

        ranked_linear, ranked_multi = design_diagnostic_digest(str(plasmid.seq), enzymes)
    except Exception as e:
        print("error", e)
        traceback.print_exc()
        return jsonify({
            "error": str(e)
        })

    return jsonify({
        "linear": ranked_linear,
        "multi": ranked_multi
        })

@app.route("/")
def main():
    return f"design diagnostic digests v{__version__}"

if __name__ == "__main__":
    app.run(host="127.0.0.1", debug=True, port=5001)
