import { useState } from "react";
import { renderToString } from "react-dom/server";
import SeqViz from "seqviz";
import seqparse from "seqparse";
import Link from "next/link";

const ThreeLaneDigest = () => {
  const [file, setFile] = useState();
  const [miniprepConc, setMiniprepConc] = useState(0);
  const [multiDigests, setMultiDigests] = useState([]);
  // const [multiDigests, setMultiDigests] = useState([
  //   { fragments: [1804, 4152], enzymes: ["AseI"] },
  //   { fragments: [2376, 3580], enzymes: ["BsaAI"] },
  //   { fragments: [1696, 4260], enzymes: ["BamHI", "EcoRI"] },
  // ]);
  const [selectedMultiDigest, setSelectedMultiDigest] = useState("n/a");
  const [linearDigests, setLinearDigests] = useState([]);
  // const [linearDigests, setLinearDigests] = useState([
  //   { fragments: [5956], enzymes: ["BamHI"] },
  //   { fragments: [5956], enzymes: ["EcoRI"] },
  //   { fragments: [5956], enzymes: ["EcoRV"] },
  // ]);
  const [selectedLinearDigest, setSelectedLinearDigest] = useState("n/a");
  const [dnaPerWell, setDnaPerWell] = useState(750);

  const [seqvizData, setSeqvizData] = useState();

  const [enzymeList, setEnzymeList] = useState(
    "ApaLI,AseI,BamHI,BsaAI,EcoRI,EcoRV,BsaI,BsmBI"
  );

  const submitRequest = async (file) => {
    const reader = new FileReader();
    reader.onload = async (e) => {
      const { name, type, seq, annotations } = await seqparse(reader.result);
      setSeqvizData({ name, seq, annotations });
      // console.log(e.target.result)
    };
    reader.readAsText(file);

    const formData = new FormData();
    formData.append("plasmid", file, file.name);
    formData.append("enzymes", enzymeList);

    try {
      const resp = await fetch("/api/design", {
        method: "POST",
        body: formData,
      });

      if (resp.ok) {
        const data = await resp.json();
        setSelectedLinearDigest("n/a");
        setSelectedMultiDigest("n/a");
        setLinearDigests(data["linear"]);
        setMultiDigests(data["multi"]);
        console.log(data);
      } else {
        throw new Error(await resp.text());
      }
    } catch (e) {
      console.error(e);
    }
  };

  const handleFileChange = async (e) => {
    if (!e.target.files) {
      return;
    }

    const file = e.target.files[0];
    if (!file) {
      return;
    }

    setFile(file);
    await submitRequest(file);
  };

  let water = "-";
  let plasmid = "-";
  let buffer = "-";
  let controlTotal = "-";
  let linearTotal = "-";
  let multiTotal = "-";
  let linear = undefined;
  let multi = undefined;

  if (selectedLinearDigest && selectedLinearDigest !== "n/a") {
    linear = JSON.parse(selectedLinearDigest);
  }

  if (selectedMultiDigest && selectedMultiDigest !== "n/a") {
    multi = JSON.parse(selectedMultiDigest);
  }

  if (
    miniprepConc &&
    miniprepConc > 0 &&
    selectedMultiDigest &&
    selectedMultiDigest !== "n/a" &&
    selectedLinearDigest &&
    selectedLinearDigest !== "n/a"
  ) {
    buffer = 2;
    plasmid = Math.ceil(dnaPerWell / miniprepConc); // 400 ng
    water = 20 - buffer - plasmid - multi.enzymes.length;

    controlTotal = water + plasmid + buffer;
    linearTotal = water + plasmid + buffer + linear.enzymes.length;
    multiTotal = water + plasmid + buffer + multi.enzymes.length;
  }

  const copyToClipboard = (
    water,
    buffer,
    plasmid,
    linear,
    multi,
    controlTotal,
    linearTotal,
    multiTotal
  ) => {
    const styles = `
    td {
      border: 1px solid black;
      padding: 2px;
    }

    p {
      margin: 0;
    }

    table {
      border-collapse: collapse;
      font-family: monospace;
    }
    `;
    const table = (
      <>
        <style>{styles}</style>
        <table>
          <tr>
            <td colSpan={3}>
              <p style={{ textAlign: "center", fontStyle: "oblique" }}>
                {file?.name.substr(0, file.name.lastIndexOf(".")) ||
                  "no file selected"}
              </p>
            </td>
          </tr>
          <tr>
            <td>{water}</td>
            <td>{water}</td>
            <td>{water}</td>
          </tr>
          <tr>
            <td>{buffer}</td>
            <td>{buffer}</td>
            <td>{buffer}</td>
          </tr>
          <tr>
            <td>{plasmid}</td>
            <td>{plasmid}</td>
            <td>{plasmid}</td>
          </tr>
          <tr>
            <td>-</td>
            <td>
              {linear ? linear.enzymes.map((e) => <p key={e}>{e}</p>) : "-"}
            </td>
            <td>
              {multi ? multi.enzymes.map((e) => <p key={e}>{e}</p>) : "-"}
            </td>
          </tr>
          <tr>
            <td>{linear ? `${linear.fragments[0]}*` : "-"}</td>
            <td>{linear ? linear.fragments[0] : "-"}</td>
            <td>
              {multi ? multi.fragments.map((f) => <p key={f}>{f}</p>) : "-"}
            </td>
          </tr>
          <tr>
            <td>{controlTotal}</td>
            <td>{linearTotal}</td>
            <td>{multiTotal}</td>
          </tr>
        </table>
      </>
    );

    const clipboardItem = new ClipboardItem({
      "text/plain": new Blob([""], { type: "text/plain" }),
      "text/html": new Blob([renderToString(table)], { type: "text/html" }),
    });

    navigator.clipboard.write([clipboardItem]);
  };

  return (
    <div>
      <table>
        <colgroup>
          <col span="1" className="noselect" />
        </colgroup>
        <tbody>
          <tr>
            <td></td>
            <td colSpan={3}>
              <input type="file" onChange={handleFileChange} /> <br />
              <div className="noprint">
                <label htmlFor="linear">Linear: </label>
                <select
                  id="linear"
                  value={selectedLinearDigest}
                  onChange={(e) => setSelectedLinearDigest(e.target.value)}
                >
                  <option key="n/a" value="n/a">
                    select a digest
                  </option>
                  {linearDigests &&
                    linearDigests.map((d, i) => (
                      <option key={i} value={JSON.stringify(d)}>
                        {d.enzymes.join(",")}: {d.fragments.join(",")}
                      </option>
                    ))}
                </select>
                <br />
                <label htmlFor="multicut">Multi-cut: </label>
                <select
                  id="multicut"
                  value={selectedMultiDigest}
                  onChange={(e) => setSelectedMultiDigest(e.target.value)}
                >
                  <option key="n/a" value="n/a">
                    select a digest
                  </option>
                  {multiDigests &&
                    multiDigests.map((d, i) => (
                      <option key={i} value={JSON.stringify(d)}>
                        {d.enzymes.join(",")}: {d.fragments.join(",")}
                      </option>
                    ))}
                </select>
                <br />
                <label htmlFor="conc">miniprep conc (ng/uL):</label>
                <input
                  id="conc"
                  type="number"
                  min="1"
                  value={miniprepConc}
                  onChange={(e) => setMiniprepConc(e.target.value)}
                />
                <br />
                <label htmlFor="dnaPerWell">DNA per well (ng): ~</label>
                <input
                  id="dnaPerWell"
                  type="number"
                  min="1"
                  style={{ width: "8ch" }}
                  value={dnaPerWell}
                  onChange={(e) => setDnaPerWell(e.target.value)}
                />
                <p>1 uL of each enzyme (10 U/uL)</p>
                <label htmlFor="enzymeList"></label>
                <textarea
                  id="enzymeList"
                  value={enzymeList}
                  style={{ width: "100%" }}
                  onChange={(e) => setEnzymeList(e.target.value)}
                />
                <p>
                  valid enzyme list:{" "}
                  <a href="/enzymes" target="_blank">
                    link
                  </a>
                </p>
                <button onClick={() => submitRequest(file)}>resubmit</button>
              </div>
            </td>
          </tr>
          <tr>
            <td></td>
            <td>supercoiled</td>
            <td>linear</td>
            <td>multi-cut</td>
          </tr>
          <tr>
            <td className="noselect"></td>
            <td colSpan={3}>
              <p style={{ textAlign: "center", fontStyle: "oblique" }}>
                {file?.name.substr(0, file.name.lastIndexOf(".")) ||
                  "no file selected"}
              </p>
            </td>
          </tr>
          <tr>
            <td>w</td>
            <td>{water}</td>
            <td>{water}</td>
            <td>{water}</td>
          </tr>
          <tr>
            <td>b</td>
            <td>{buffer}</td>
            <td>{buffer}</td>
            <td>{buffer}</td>
          </tr>
          <tr>
            <td>p</td>
            <td>{plasmid}</td>
            <td>{plasmid}</td>
            <td>{plasmid}</td>
          </tr>
          <tr>
            <td>e</td>
            <td>-</td>
            <td>
              {linear ? linear.enzymes.map((e) => <p key={e}>{e}</p>) : "-"}
            </td>
            <td>
              {multi ? multi.enzymes.map((e) => <p key={e}>{e}</p>) : "-"}
            </td>
          </tr>
          <tr>
            <td>len</td>
            <td>{linear ? `${linear.fragments[0]}*` : "-"}</td>
            <td>{linear ? linear.fragments[0] : "-"}</td>
            <td>
              {multi ? multi.fragments.map((f) => <p key={f}>{f}</p>) : "-"}
            </td>
          </tr>
          <tr>
            <td>tot</td>
            <td>{controlTotal}</td>
            <td>{linearTotal}</td>
            <td>{multiTotal}</td>
          </tr>
        </tbody>
      </table>
      <button
        onClick={() =>
          copyToClipboard(
            water,
            buffer,
            plasmid,
            linear,
            multi,
            controlTotal,
            linearTotal,
            multiTotal
          )
        }
      >
        copy to clipboard
      </button>

      <div>
        {seqvizData && (
          <SeqViz
            name={seqvizData.name}
            seq={seqvizData.seq}
            annotations={seqvizData.annotations}
            style={{
              height: "900px",
              width: "900px",
            }}
            viewer="circular"
            rotateOnScroll={false}
            enzymes={multi?.enzymes || []}
          />
        )}
      </div>
    </div>
  );
};

export default ThreeLaneDigest;
